#-----------------------------------------------------------------------------
# Sparsifier
# Idea by Tguyus
# Copyright (c) 2004 David G. Drumright (ockham)
# Oct 26, 2004
# Simplified for Mac use 12/20/06.

# Highlight desired body parts in the list; or to apply to all parts,
# just leave the list alone.

# When you hit Go, the script removes every other keyframe
# on the selected parts, but only if that keyframe is "unnecessary" in the
# sense that it could just as well be splined.  If this frame is above both
# neighbors or below both neighbors, it remains.

#Applying the script repeatedly will sparsify even more.  (The second run seems to simplify about as far
# as generally needed.)

# Note that the listbox has special entries for "full right hand" and "full left hand".
# If you want to sparsify either hand with its fingers, use these.  If you want
# to pick out specific finger joints, you'll still find them further down in the list.
#-----------------------------------------------------------------------------

import poser
scene = poser.Scene()

TheFig=scene.CurrentFigure()
if not TheFig: raise "No figure selected"

#-----------------------------------------------------------------------------

def Sparsify(ActorName):
    # Go through all keyframes.  Cut out odd-numbered frames that are
    # not "necessary".  Specifically, if the value at the odd-numbered frame is
    # between the values of its neighboring evens, eliminate it.

    # To avoid possible interference between the changes and the actual,
    # build an array of the frames to delete first; then go through and delete them.
    TheActor=None
    try:
        TheActor=TheFig.Actor(ActorName)
    except:
        pass
    if not TheActor: return

    for OneParm in TheActor.Parameters():
        if OneParm.Hidden(): continue
        Last=scene.NumFrames()-1
        ActualFrames=[]
        NeedsDelete=[]

        # Begin by getting a list of the actual keyframes so we can work
        # on that basis instead of raw frames.  This will make it possible
        # to use the script repeatedly.
        F=1
        while F and (F < Last):
            scene.SetFrame(F)
            ActualFrames.append(F)
            F=OneParm.NextKeyFrame(F)

        # Now do the real run through these frames.
        for F in ActualFrames:
            scene.SetFrame(F)
            ThisVal=OneParm.Value()

            # Look at its neighbors.  Check to be sure
            # we're not beyond ends.
            try:
                Prev=OneParm.PrevKeyFrame()
            except:
                continue
            try:
                Next=OneParm.NextKeyFrame()
            except:
                break
            if not Next: break
            if Next >= Last: break

            # After testing, get the values:
            scene.SetFrame(Prev)
            PrevVal=OneParm.Value()
            scene.SetFrame(Next)
            NextVal=OneParm.Value()

            Keep=0 # Var not really used, just makes the Ifs easier to edit.
            if   (ThisVal > PrevVal) and (ThisVal > NextVal): Keep=1  # Local max
            elif (ThisVal < PrevVal) and (ThisVal < NextVal): Keep=1  # Local min
            else:
                NeedsDelete.append(F)

        # Now go through and use the list. Reminder: at this point
        # in the indents, we are still in the same parameter of the same actor.
        for F in NeedsDelete:
            scene.SetFrame(F)
            OneParm.DeleteKeyFrame(F)


#------------------------------------------------------------
for A in TheFig.Actors():
    Sparsify(A.Name())
#-----------------------------------------------------------------------------
