#-----------------------------------------------------------------------------
# Sparsifier
# Idea by Tguyus
# Copyright (c) 2004 David G. Drumright (ockham)
# Oct 26, 2004

# Highlight desired body parts in the list; or to apply to all parts,
# just leave the list alone.

# When you hit Go, the script removes every other keyframe
# on the selected parts, but only if that keyframe is "unnecessary" in the
# sense that it could just as well be splined.  If this frame is above both
# neighbors or below both neighbors, it remains.

#Applying the script repeatedly will sparsify even more.  (The second run seems to simplify about as far
# as generally needed.)

# Note that the listbox has special entries for "full right hand" and "full left hand".
# If you want to sparsify either hand with its fingers, use these.  If you want
# to pick out specific finger joints, you'll still find them further down in the list.
#-----------------------------------------------------------------------------

from Tkinter import *
import poser
scene = poser.Scene()

TheFig=scene.CurrentFigure()
if not TheFig: raise Exception("No figure selected")

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
  def __init__(self, master, textMessage):
    self.master = master
    master.title("Sparsifier")

    self.StatusEntry = Entry(self.master,width=30)
    self.StatusEntry.grid(row=0,column=0,columnspan=2)
    self.StatusEntry.insert(0,"Pick parts, hit Go")

    # Lists
    self.ListScroll = Scrollbar(self.master, orient=VERTICAL)
    self.ListScroll.grid( row=2, column=0,sticky=N+S+E)
    self.List = Listbox(self.master, height=25,width=55, selectmode=MULTIPLE,yscrollcommand = self.ListScroll.set,exportselection=0)
    self.List.grid( row=2, column=1)
    # Set the scrollbar to work
    self.ListScroll["command"] = self.List.yview

    # Go/nogo buttons.
    self.buttonGo = Button(self.master, text="Sparsify", command=self.HandleGo)
    self.buttonGo.grid(row=3, column=0)

    self.buttonClear = Button(self.master, text="Clear", command=self.HandleClear)
    self.buttonClear.grid(row=3, column=1)


    self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

    # Fill the list with body parts of current figure, with special Hand entries.
    self.List.insert(END,"Full Hand Left")
    self.List.insert(END,"Full Hand Right")
    for OneActor in TheFig.Actors():
      self.List.insert(END,OneActor.Name())

  # - - - - - - - - - - - - - - - - - -
  def ShowStatus(self,S): # Just saving repetition
    self.StatusEntry.delete(0,END)
    self.StatusEntry.insert(0,S)
    self.StatusEntry.update_idletasks()

  # - - - - - - - - - - - - - - - - - -

  def Update(self):
    if scene: scene.ProcessSomeEvents(1)
    root.lift()
    root.after(100, self.Update)

  # - - - - - - - - - - - - - - - - - -
  # - - - - - - - - - - - - - - - - - -

  def HandleGo(self):
    Sels=self.List.curselection()
    if not Sels:
      # Do all the items except the special FullLeft and FullRight, which are the first two.
      for i in range(2,self.List.size()):
        PartName = self.List.get(i)
        self.Sparsify(PartName)
    else:
      for S in Sels:
        PartName = self.List.get(int(S))
        if   PartName=="Full Hand Left":  self.SparsifyFullHandLeft()
        elif PartName=="Full Hand Right": self.SparsifyFullHandRight()
        else:               self.Sparsify(PartName)

    scene.SetFrame(0)

  # - - - - - - - - - - - - - - - - - -
  def HandleClear(self):
    Sels=self.List.curselection()
    if not Sels:
      # Do all the items except the special FullLeft and FullRight, which are the first two.
      for i in range(2,self.List.size()):
        PartName = self.List.get(i)
        self.Clear(PartName)
    else:
      for S in Sels:
        PartName = self.List.get(int(S))
        if   PartName=="Full Hand Left":  self.ClearFullHandLeft()
        elif PartName=="Full Hand Right": self.ClearFullHandRight()
        else:               self.Clear(PartName)

    scene.SetFrame(0)

  # - - - - - - - - - - - - - - - - - -

  def Sparsify(self,ActorName):
    # Go through all keyframes.  Cut out odd-numbered frames that are
    # not "necessary".  Specifically, if the value at the odd-numbered frame is
    # between the values of its neighboring evens, eliminate it.

    # To avoid possible interference between the changes and the actual,
    # build an array of the frames to delete first; then go through and delete them.
    try:
      TheActor=TheFig.Actor(ActorName)
    except:
      return
    if not TheActor: return
    for OneParm in TheActor.Parameters():
      if OneParm.Hidden(): continue
      self.ShowStatus("working on actor %s parm %s" % (ActorName,OneParm.Name()) )
      Last=scene.NumFrames()-1
      ActualFrames=[]
      NeedsDelete=[]

      # Begin by getting a list of the actual keyframes so we can work
      # on that basis instead of raw frames.  This will make it possible
      # to use the script repeatedly.
      F=1
      while F and (F < Last):
        scene.SetFrame(F)
        ActualFrames.append(F)
        F=OneParm.NextKeyFrame(F)

      # Now do the real run through these frames.
      for F in ActualFrames:
        scene.SetFrame(F)
        ThisVal=OneParm.Value()

        # Look at its neighbors.  Check to be sure
        # we're not beyond ends.
        try:
          Prev=OneParm.PrevKeyFrame()
        except:
          continue
        try:
          Next=OneParm.NextKeyFrame()
        except:
          break
        if not Next: break
        if Next >= Last: break

        # After testing, get the values:
        scene.SetFrame(Prev)
        PrevVal=OneParm.Value()
        scene.SetFrame(Next)
        NextVal=OneParm.Value()

        Keep=0 # Var not really used, just makes the Ifs easier to edit.
        if   (ThisVal > PrevVal) and (ThisVal > NextVal): Keep=1  # Local max
        elif (ThisVal < PrevVal) and (ThisVal < NextVal): Keep=1  # Local min
        else:
          NeedsDelete.append(F)

      # Now go through and use the list. Reminder: at this point
      # in the indents, we are still in the same parameter of the same actor.
      for F in NeedsDelete:
        scene.SetFrame(F)
        OneParm.DeleteKeyFrame(F)
    self.ShowStatus("Done")
  # - - - - - - - - - - - - - - - - - -

  def SparsifyFullHandLeft(self):
    # Just use all the standard names for left hand, and run the sparsify for all
    LParts = ["lHand",
          "lThumb1",  "lThumb2",  "lThumb3",
          "lIndex1",  "lIndex2",  "lIndex3",
          "lMid1",    "lMid2",    "lMid3",
          "lRing1",   "lRing2",   "lRing3",
          "lPinky1",  "lPinky2",  "lPinky3"]
    for OneActor in LParts:
      self.Sparsify(OneActor)

  # - - - - - - - - - - - - - - - - - -

  def SparsifyFullHandRight(self):
    # Just use all the standard names for right hand, and run the sparsify for all
    RParts = ["rHand",
          "rThumb1", "rThumb2", "rThumb3",
          "rIndex1", "rIndex2", "rIndex3",
          "rMid1",   "rMid2",   "rMid3",
          "rRing1",  "rRing2",  "rRing3",
          "rPinky1", "rPinky2", "rPinky3"]
    for OneActor in RParts:
      self.Sparsify(OneActor)

  # - - - - - - - - - - - - - - - - - -

  def Clear(self,ActorName):
    # Go through all keyframes.  Cut out all frames.
    try:
      TheActor=TheFig.Actor(ActorName)
    except:
      return
    if not TheActor: return
    self.ShowStatus("working on actor %s" % ActorName)
    for f in range(scene.NumFrames()):
      scene.SetFrame(f)
      TheActor.DeleteKeyFrame(f)  # get rid of any keyframes for it.
    self.ShowStatus("Done")

  # - - - - - - - - - - - - - - - - - -

  def ClearFullHandLeft(self):
    # Just use all the standard names for left hand, and run the sparsify for all
    LParts = ["lHand",
          "lThumb1",  "lThumb2",  "lThumb3",
          "lIndex1",  "lIndex2",  "lIndex3",
          "lMid1",    "lMid2",    "lMid3",
          "lRing1",   "lRing2",   "lRing3",
          "lPinky1",  "lPinky2",  "lPinky3"]
    for OneActor in LParts:
      self.Clear(OneActor)

  # - - - - - - - - - - - - - - - - - -

  def ClearFullHandRight(self):
    # Just use all the standard names for right hand, and run the sparsify for all
    RParts = ["rHand",
          "rThumb1", "rThumb2", "rThumb3",
          "rIndex1", "rIndex2", "rIndex3",
          "rMid1",   "rMid2",   "rMid3",
          "rRing1",  "rRing2",  "rRing3",
          "rPinky1", "rPinky2", "rPinky3"]
    for OneActor in RParts:
      self.Clear(OneActor)
  # - - - - - - - - - - - - - - - - - -

  def HandleCancel(self):
    self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
